/* Télémesure et commande d'un PA DATV pour QO100 par liaison ethernet (F6AAM)
 * *  VERSION 6.0 du 7/12/2020 *** 
 * 
 *  MOTIVATIONS : 
 *  
 *  Afin de réduire au maximum la longueur du coax entre la sortie antenne du PA et la parabole (le PA est situé à 23m du shack), 
 *  il est apparu nécessaire de piloter et surveiller ce PA à distance.
 *  Une solution "élégante" comme celle décrite par F6DZP (http://www.vivadatv.org/viewtopic.php?f=87&t=698) et que j'avais également repérée sur le net, 
 *  consiste à utiliser une liaison ethernet filaire avec un Arduino comme serveur et une page web qui tourne sur un navigateur coté client (le shack)
 *  
 *  VERSIONS : 
 *  
 *  Modif par rapport version 0.0 du 1/11/2019
 *  version 1.0
 *  - Ajout du stockage en EEPROM des limites de température haute, basse, U_PA, I_PA, Dir_RF, Refl_RF 
 *  version 2.0
 *  - Ajout d'un watchdog coté serveur et coté client qui contrôlent l'intégrité de la laison ethernet sous forme d'un message d'avertissement de "liaison ethernet : !!! OFF !!!" 
 *    En fonctionnement le message indique alternativement "liaison ethernet ON" puis "ilaison ethernet on"   
 *  version 3.0
 *  - Les limites sont maintenant gérées sur la carte arduino et non plus coté client (amélioration de la rapidité de réaction en cas d'alarme).
 *  version 4.0
 *  - Un compteur indique approximativement la durée totale des coupures ethernet (remise à zéro possible avec le bouton reset).
 *  - Possibilité de configurer dynamiquement une option qui coupe les relais PTT et PA si un défaut de liaison ethernet intervient
 *  version 4.1
 *  - Ajout de la commande d'un 4ème relais permettant de piloter une sortie auxiliaire (comme le PLUTO par F1TE Radio REF d'avril 2020)
 *  version 5 du 13/5/2020
 *  - Ajout d'une procédure simplifiée pour etalonner directement sur la page web, les mesures de tension, courant et puissances sans nécessité de recompiler à chaque mesure.
 *    Les valeurs de calibration sont alors stockées en EEPROM.
 *  version 5.1 du 16/5/2020
 *  - Simplification de la calibration des puissances HF.  
 *  - Possibilité de modifier les valeurs des atténuateurs et valeurs de couplage des 2 voies de puissance directe et réfléchie
 *  version 5.2 du 8/11/2020
 *  - chargement de la page Web 2 fois plus rapide
 *  *  version 6.0 du 7/12/2020
 *  - possibilité de charger 3 versions différentes :
 *  version Longue qui contient tout (indispensable pour la mise au point comme la calibration des mesures)
 *  version Medium qui contient tout sauf les calibrations (qui sera utilisée le plus souvent une fois les calibrations effectués)
 *  version Courte qui contient uniquement l'affichage des mesures (interessant pour intégrer dans le contrôleur du Pluto par exemple (chargement en 2 secondes)
 *  Le choix de la version chargée au prochain lancement peut être effectué sur chacune des 3 pages web.
 *  A la première mise sous tension ou suite à un reset de la carte Arduino la version Medium sera chargée par défaut.
 *  2 versions peuvent cohabiter sans perturbations du fonctionnement (la medium et la courte par exemple)
 *
 *  MATERIEL :
 *  
 *  Carte Arduino MEGA 2560 R3 (la carte UNO n'a pas assez de mémoire ram !)
 *  Shield ethernet arduino avec lecteur de carte SD
 *  Module 4 relais arduino
 *  Module sonde de température DS18B20 (communication de type "One Wire")
 *  2 modules AD 8318 (ampli logaritmique ANALOG DEVICE pour lecture des puissances directe et réfléchie en dBm )
 *  1 coupleur directif 40dB pour la mesure du réfléchi (mon PA (Axis-NT) a déjà un coupleur intégré pour la mesure du direct)
 *  1 module de mesure de courant à effet Hall ACS758
 *  2 switch tp-link model TL-SF1005D ou équivalent
 *  Câble ethernet catégorie 5 ou mieux 6
 *   
 *  
 *  LOGICIEL :
 *  
 *  Je n'ai pas non plus inventé la roue, j'ai juste fait tourné ensemble plusieurs méthodes puisées sur le net :
 *  
 *  Utilisation de la maquette du logiciel de communication ethernet utilisant les requêtes AJAX 
 *  décrit ici :  https://startingelectronics.org/tutorials/arduino/ethernet-shield-web-server-tutorial/SD-card-IO/ 
 *  Le grand interet des commandes AJAX est de permettre un échange de données entre le serveur et le client avec des messages très courts 
 *  sans avoir besoin de recharger la page Web à chaque mesure.
 *  
 *  Utilisation du logiciel de lecture du module de température DS18B20
 *  décrit ici :  https://www.carnetdumaker.net/articles/mesurer-une-temperature-avec-un-capteur-1-wire-ds18b20-et-une-carte-arduino-genuino/ 
 *  
 *  Utilisation d'une partie du logiciel de mesure de puissance avec les modules AD 8318
 *  décrit ici : article Radio-Ref 936 novembre 2019 Mesure de la puissance directe, réfléchie et ROS
 *  code source ici : http://urls.r-e-f.org/ij691wo
 *    
 *  Utilisation de la mise en place d'un watchdog coté Arduino décrit ici : https://www.youtube.com/watch?v=BDsu8YhYn8g
 *  
 *  Utilisation de l'article pour stocker des données en EEPROM sur Arduino décrit ici : 
 *  https://www.carnetdumaker.net/articles/stocker-des-donnees-en-memoire-eeprom-avec-une-carte-arduino-genuino/
 */

/* PRINCIPE GENERAL : 
 *  
 *  l'arduino est programmé en tant que serveur. Le client réside sur un PC, Tablette ou smartphone.
 *  A la première connexion d'un "client" (appel du client en tapant l'adresse du serveur sur n'importe quel navigateur), 
 *  le serveur (l'arduino) répond en envoyant au "client" l'unique page web (le fichier index.htm) préalablement chargé sur la carte SD résident sur le shield Arduino.
 *  Une fois chargée sur le navigateur la page index.htm s'exécute et intérroge le serveur Arduino toutes les secondes environ.
 *  
 *  La première tâche du serveur est la récupération des seuils d'alarme stockés dans l'EEPROM de l'ARDUINO pour les envoyer au client.
 *  Le client pourra ensuite modifier ces seuils comme il le souhaite, le serveur non seulement en tiendra compte en temps réel 
 *  mais les stockera également dans l'EEPROM pour qu'ils apparaissent ainsi modifiés à la prochaine utilisation.
 *  
 *  A chaque requête reçue le serveur mesure et envoie la température, la tension du PA (+28V), l'intensité consommée par le PA, 
 *  La puissance HF directe si mesurable et la puissance HF indirecte si mesurable.
 *  
 *  Le serveur execute également les 4 commandes envoyées coté client c'est à dire la mise ON ou OFF du ventilateur(FAN), de la tension sur le PA et le relais PTT et un relais Auxiliaire. 
 *  La logique des conditions d'exécution des 3 premières  commandes est gérée dans la page web du navigateur en Javascrip.
 *  Ainsi il n'est pas possible par exemple de mettre "ON" le PA et le PTT si le ventilateur n'est pas "ON"
 *  D'une manière générale les boutons non autorisés sont visibles mais grisés et anactifs.
 *  
 *    
 *  entrées analogiques : A0 et A1 déjà utilisées par l'ethernet shield !
 *  A2 : pont diviseur rapport 1/6 pour mesure de la tension du PA (U_PA)
 *  A3 : sortie du capteur de courant à effet hall ACS758
 *  A4 : sortie du modue AD 8318 pour mesure puissance directe (utiliser des atténuateurs car le AD 8318 ne supporte pas plus de 12 dBm et la mesure n'est linéaire qu'en dessous de -5dBm !)
 *  A5 : sortie du modue AD 8318 pour mesure puissance réfléchie (utiliser des atténuateurs car le AD 8318 ne supporte pas plus de 12 dBm et la mesure n'est linéaire qu'en dessous de -5dBm !)
 *  Personellement j'utilise 2 atténuateurs (30 + 20 dB) pour la mesure du directe (l'ampli AXIS-NT que j'utilse fourni déjà une sortie du directe à -38dB)
 *  J'utile la sortie -40dB d'un coupleur directif pour le réfléchi ce qui m'oblige de rajouter un atténuateur de 20dB pour l'entrée de l'AD 8318.
 *  
 *  QUELQUES PIEGES ET REMEDES : 
 *  
 * 1 Le module shield ethernet ne semble pas répondre même à un simple ping :
 *   - Assurez vous que l'adresse IP 192.168.1.23 choisie et programmée par défaut ne rentre pas en conflit avec un autre équipement local ayant la même adresse !
 *   - Utilisez un logiciel gratuit de scan d'adresses IP comme dipiscan par exemple (il en existe des tas d'autres ...)
 *   - Une fois l'adresse IP trouvée ne pas oublier de la modifier dans ce source à la ligne : IPAddress ip(192, 168, 1, 23); // IP address, may need to change depending on network
 *    
 *   - Certaines cartes (chinoises) sont livrées avec un réseau de résistances de 510 ohms au lieu de 50 ohms pour adapter l'impédance du cable ethernet.
 *     Pour y remédier il vous suffit de rajouter 2 résistances de 120 ou 100 ohms comme décrit ici : https://forum.arduino.cc/index.php?topic=447408.0
 *    
 * 2 Le module shield ne démarre pas tout seul à la mise sous tension et nécessite un reset manuel :
 *   - pour celà rajouter un réseau RC comme signalé par F1TE ici : http://tigawd.blogspot.com/2015/05/arduino-uno-clone-and-w5100-ethernet.html  
 * 
 * 3 La liaison se déconnecte trop souvent (apparition de message "liaison ethernet : !!! OFF !!!")  
 *   - Veuillez à utiliser uniquement une liaison filaire et non WiFi ce qui serait un non sens vu l'emission à 2.4Ghz avec quelques dizaines de Watts ! 
 *   - Veiller à n'établir qu'une seule connexion à la fois (en d'autres termes n'utilisez pas plusieurs navigateurs sur l'adresse de l'arduino simultanément)
 *   - Utilisez un câble ethernet blindé catégorie 5 voire 6.
 *   - Enfin utilisez de chaque coté un switch ce qui a l'avantage d'amplifier le signal et d'utiliser un câble droit ethernet.
 *   - Personnellement j'utilise 2 switch de marque tp-link model TL-SF1005D.
 *    
 * 4 Pas de chargement du fichier index.htm suite à la réinsertion de la carte SD consécutive par exemple à une modification du fichier index.htm
 *   - Après chaque réinsertion de la carte SD sur le réceptacle de la carte shield, il faut systématiquement appuyer sur le bouton reset de la carte SD.
 *   - Certains PC ou MAC ne fonctionnent pas en liaison filaire et WIFI simultanément : pensez à désactiver la liason WI-FI
 *    
 * 5 Plein d'erreurs à la compilation du projet.ino
 *    - il ne doit y avoir qu'un seul fichier .ino sous le repertoire qui doit d'ailleurs porter le même nom que le fichier .ino
 *    
 * 6 En cas de non utilisation de mesure des puissance (par absence des catre AD 8318 par exemple) il est impératif d'envoyer du + 5V sur les entrées A4 et A5 à travers 1 à 10K.
 *   En effet ces cartes ont une réponse en pente négative ce qui donne 2 V en sortie pour -60dBm (1nW) en entrée et 0,6V en sortie pour 0 dBm (1mW) en entrée.
 *   En l'absence de carte le logiciel interpréterait les 0V de ses entrées A4 et A5 comme une puissance de 0dBm soit compte tenu des atténuateurs mis en place,
 *   des puissances de 68dBm pour la puissance directe et 60dBm pour le réfléchi !!!!
 *    
 */
 
#include <avr/wdt.h>  // pour le wachdog
#include <EEPROM.h>   // pour stocker les limites d'alarmes
#include <SPI.h>      // pour piloter la carte shield ethernet et la carte SD
#include <Ethernet.h> // pour la carte ethernet
#include <SD.h>       // pour la carte SD
#include <OneWire.h>  // pour piloter la sonde de température


// ---------- Initialisation des variables --------------------
volatile long ii,jj;
/************** déclarations propres au DS18B20********************/
const int DS18B20_PIN = 40;
const int DS18B20_ID=0x28;
// Déclaration de l'objet ds
OneWire ds(DS18B20_PIN); // on pin DS18B20_PIN (a 4.7K resistor is necessary)

// Variables générales
float DS18B20_temperature;

// variable pour indique un étalonnage de puissances HF en cours
//boolean calib_D_P_HF = false;
boolean calib_R_P_HF = false;
//boolean Etalonnage_U_PA = false;
float G_U_PA;
//boolean Etalonnage_I_PA = false;
float G_I_PA;
float zero_I;       // pour lire 0 quand le courant est nul
float G1;           // correction gain voie puissance directe
float OFFSET1;      // correction offset voie puissance directe
float G2;           // correction gain voie puissance réfléchie
float OFFSET2;      // correction offset voie puissance réfléchie

/* Résolution disponibles */
const byte RESOLUTION_12_BITS = 0b01111111; // 800 ms
const byte RESOLUTION_11_BITS = 0b01011111; // 400 ms
const byte RESOLUTION_10_BITS = 0b00111111; // 200 ms
const byte RESOLUTION_9_BITS = 0b00011111;  // 100 ms

/******************************************************************/

// ----------- variables propres à la mesure des puissances HF 
int iii;
byte att_CH1 , att_CH1_prev, att_CH2 = 0 , att_CH2_prev ;
int   error_limit_LOW , error_limit_HIGH; 
float f_ghz,  mmm,  ccc ;
float level_CH_1;
float level_CH_2;
float Corr_SR;
float seuil_R_P_HF;

/******************************************************************/

// ---------déclaration des variables de type string pour les limites
char str_lim_temp_haute[5] = {0};
char str_lim_temp_basse[5] = {0};
char str_lim_U_PA[5] = {0};
char str_lim_I_PA[5] = {0};
char str_lim_Dir_RF[5] = {0};
char str_lim_Refl_RF[5] = {0};
char str_valeur[10] = {0};
char str_valeur2[10] = {0};
char str_valeur3[10] = {0};
char str_valeur4[10] = {0};

byte web_page = 2;

/******************************************************************/

// -------- déclaration de variables pour l'EEPROM
/** Le nombre magique et le numéro de version actuelle */
static const unsigned long STRUCT_MAGIC = 123456789; // si modifié le logiciel rechargera les valeurs de limites par défaut définie plus bas
// De plus au tout premier lancement du programme sur la platine Arduino, les valeurs par défaut seront chargées dans l'EEPROM qui est évidemment vide !!
static const byte STRUCT_VERSION = 5; // not used at this moment !

// déclaration stucture de données EEPROM (limites de temp haute et basse de U_PA de I_PA de Dir_RF et de Refl_RF
struct MaStructure {
  unsigned long magic;
  byte  struct_version;
  float lim_temp_haute;
  float lim_temp_basse;
  float lim_U_PA;
  float lim_I_PA;
  float lim_Dir_RF;
  float lim_Refl_RF;
  float G_U_PA;       // facteur de calibration pour la mesure de la tension U_PA
  float G_I;          // Gain pour la mesure du courant I_PA
  float OFFSET_I;     // offset pour la mesure du courant I_PA
  float pente_D;      // pente voie puissance directe
  float B_D;          // offset voie puissance directe
  float pente_R;      // pente voie puissance réfléchie
  float B_R;          // offset voie puissance réfléchie
  float Att_D;         // Atténuateur de mesure puissance directe
  float Att_R;         // Atténuateur de mesure puissance reflechie
  
};
/** déclaration instances de la structure, globales, car utilisées dans plusieurs endroits du programme */
MaStructure ms;
MaStructure ms_lue;
MaStructure ms_Adjust_HF;
/******************************************************************/
const int SERIAL_PORT=9600; // pour débogage avec le moniteur série
/******************************************************************/
// size of buffer used to capture HTTP requests
#define REQ_BUF_SZ   100
/******************************************************************/
// MAC address from Ethernet shield sticker under board
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 1, 23);   // IP address, may need to change depending on network
EthernetServer server(80);       // create a server at port 80
File webFile;                    // the web page file on the SD card
char HTTP_req[REQ_BUF_SZ] = {0}; // buffered HTTP request stored as null terminated string
char req_index = 0;              // index into HTTP_req buffer

boolean Relay_PTT = 0; // stores the states of the Relays
boolean Relay_PA = 0;
boolean Relay_FAN = 0;
boolean Relay_AUX = 0;
boolean Watchdog = 0;
boolean alarm_T_PA_Arduino = 0;
boolean alarm_U_PA_Arduino = 0;
boolean alarm_I_PA_Arduino = 0;
boolean alarm_Dir_power_Arduino = 0;
boolean alarm_Refl_power_Arduino = 0;
boolean maj_EEPROM = 0;

boolean req_lim = 0; // pour savoir si une demande des limites est faite par le client

// variables pour mémoriser la valeur max mesurée
float T_PA_max = 0;
float U_PA_max = 0;
float I_PA_max = 0;
float Dir_RF_max = 0;
float Refl_RF_max = 0;


float U_PA, I_PA;

byte count_envoi_index = 0;

//this function setups up and starts the watchdog timer
void setWDT(byte sWDT) {
   WDTCSR |= 0b00011000;
   WDTCSR = sWDT |  WDTO_2S; //Set WDT based user setting and for 2 second interval
   wdt_reset();
}

byte PA_OFF_ON_LINK_OFF = 0; // variable qui determine si le serveur coupe les relais PA et PTT sur une coupure de liaison ethernet

ISR (WDT_vect) // routine d'interruption executée si la liaison ethernet est coupée (le Watchdog n'a pas été remis à zéro dans un délai de 2 secondes)
{
  if (PA_OFF_ON_LINK_OFF == 1) { // si l'utilisateur a choisi de couper PTT et PA en cas de coupure liaison ethernet
    // on coupe le PTT 
    Relay_PTT = 0;  // save Relay state
    digitalWrite(6, HIGH); 
    // boucle d'attente programmée car la fonction delai ne marche pas dans une routine d'interruption
    for (ii = 1; ii <= 1000; ii++) {
       for (jj = 1; jj <= 500; jj++) {
       }
    }
    // puis on coupe l'alim PA 
    Relay_PA = 0;  // save Relay state
    digitalWrite(7, HIGH); 
  }  
}  // end of WDT_vect

void setup()

{
 
    cli();// disable all interrupts
    wdt_disable();  // Disable the watchdog and wait for more than 3 seconds 
    delay(3000);  // Done so that the Arduino doesn't keep resetting infinitely in case of wrong configuration 
    setWDT(0b01000000); //set watchdog for interrupt
    sei();//enable interrupts
  
    web_page = 2;

    // initialiser la page web comme medium
    
  
    // disable Ethernet chip
    pinMode(10, OUTPUT);
    digitalWrite(10, HIGH);
    
    Serial.begin(115200);       // for debugging
    
    // initialize SD card
    Serial.println("Initializing SD card...");
    if (!SD.begin(4)) {
        Serial.println("ERROR - SD card initialization failed!");
        return;    // init failed
    }
    Serial.println("SUCCESS - SD card initialized.");
    // check for index.htm file
    if (!SD.exists("index.htm")) {
        Serial.println("ERROR - Can't find index.htm file!");
        return;  // can't find index file
    }
    Serial.println("SUCCESS - Found index.htm file.");
    
    // switches on pins 2, 3 and 5
    pinMode(2, INPUT);
    pinMode(3, INPUT);
    pinMode(5, INPUT);
    // Relays
    pinMode(6, OUTPUT);
    pinMode(7, OUTPUT);
    pinMode(8, OUTPUT);
    pinMode(9, OUTPUT);
    digitalWrite(6, HIGH);
    digitalWrite(7, HIGH);
    digitalWrite(8, HIGH);
    digitalWrite(9, HIGH);
    
    Ethernet.begin(mac, ip);  // initialize Ethernet device
    server.begin();           // start to listen for clients
    
    Serial.print("\nLe serveur est sur l'adresse : ");
    Serial.println(Ethernet.localIP()); //on affiche l'adresse IP de la connexion

   
    // Charge la mémoire EEPROM
    chargeEEPROM();

}


/** Sauvegarde en mémoire EEPROM le contenu actuel de la structure */
void sauvegardeEEPROM() {
  // Met à jour le nombre magic et le numéro de version avant l'écriture
  ms.magic = STRUCT_MAGIC;
  ms.struct_version =  STRUCT_VERSION;
  //Serial.println("*******************************INITIALISATION EEPROM ****************************/");
  EEPROM.put(0, ms);
}

/** Charge le contenu de la mémoire EEPROM dans la structure */
void chargeEEPROM() {
Serial.print("STRUCT_MAGIC = ");Serial.println(STRUCT_MAGIC);

  // Lit la mémoire EEPROM
  EEPROM.get(0, ms);
  
  // Detection d'une mémoire non initialisée
  byte erreur = ms.magic != STRUCT_MAGIC;

 
  if (erreur) {
    Serial.println("ms.magic != STRUCT_MAGIC");
    // Valeurs par défaut pour le tout premier démarrage du programme 
    
    ms.lim_temp_haute = 40.0;
    ms.lim_temp_basse = 25.5;
    ms.lim_U_PA = 29.0;
    ms.lim_I_PA = 11.0;
    ms.lim_Dir_RF = 50.50;     // > 100W
    ms.lim_Refl_RF = 33.0;    // 2W
    ms.G_U_PA = 3.09;         // Gain pour la mesure de la tension U_PA
    ms.G_I = 8.22;          // Gain pour la mesure du courant I_PA
    ms.OFFSET_I = 514.54;      // offset pour la mesure du courant I_PA
    ms.pente_D = 25.95;       // pente voie puissance directe
    ms.B_D = 461.49;          // offset voie puissance directe
    ms.pente_R = 26.18;       // pente voie puissance réfléchie
    ms.B_R = 480.85;          // offset voie puissance réfléchie
    ms.Att_D = 68.0;          // Atténuateur de mesure puissance directe
    ms.Att_R = 57.0;          // Atténuateur de mesure puissance reflechie
    // Sauvegarde les nouvelles données
    sauvegardeEEPROM();
  }

}

void mesures() {

    //Serial.println("*************************DEBUT MESURES*********************************");
   
    // lire les données stockées en EEPROM
    EEPROM.get(0, ms_lue);
    // mesure de la température
    DS18B20_temperature = getTemperatureDS18b20(); // On lance la fonction d'acquisition de T°
    if (DS18B20_temperature >  ms_lue.lim_temp_haute) { // on ne veut pas dépasser le seuil de Température

      T_PA_max = DS18B20_temperature; // on garde la valeur qui a dépassé le seuil max
      // on coupe le PTT immédiatement 
      Relay_PTT = 0;  // save Relay state
      digitalWrite(6, HIGH); // on coupe le relais PTT
      delay(500); // on temporise pour ne pas couper le relai PA au maximum de courant
      Relay_PA = 0;  // save Relay state 
      digitalWrite(7, HIGH); // puis on coupe le relais PA
      alarm_T_PA_Arduino = 1;
      //Serial.println("ordre coupure PTT puis PA Power car dépassement Température");   
    }
      
    // mesure de la tension alim du PA
    float U;
    U = analogRead(2); // on mesure la tension U sur A2
    for(iii=0;iii<49; iii++)                 // iii = counter index      
        U  =  U + analogRead(2);       // on fait 49 lectures
    U  = U/50 ;                        // on fait la moyenne
    U_PA = (U * ms_lue.G_U_PA)/100;       // formule issue de l'EEPROM
    if (U_PA > ms_lue.lim_U_PA) {            // on ne veut pas dépasser le seuil de tension PA
      U_PA_max = U_PA;                       // on garde la valeur qui a dépassé le seuil max
      // on coupe l'alim PA immédiatement 
      Relay_PTT = 0;  // save Relay state
      digitalWrite(6, HIGH); // on coupe le relais PTT
      delay(500);  // on temporise pour ne pas couper le relai PA au maximum de courant
      Relay_PA = 0;  // save Relay state 
      digitalWrite(7, HIGH); // puis on coupe le relais PA
      alarm_U_PA_Arduino = 1;
      //Serial.println("ordre coupure PTT puis PA Power car U_PA > seuil client");    
    }
    
    // mesure du courant alim du PA (I_PA)
    float I;
    I = analogRead(3); // on mesure le courant I_PA sur A3
    for(iii=0;iii<199; iii++)                       // iii = counter index      
        I  =  I + analogRead(3);              // on fait 199 lectures
    I  = I/200 ;                              // on fait la moyenne

    //Serial.print("ms_lue.OFFSET_I = ");
    //Serial.println(ms_lue.OFFSET_I);  
    //Serial.print("ms_lue.G_I = ");
    //Serial.println(ms_lue.G_I);  
    //Serial.print("I = ");
    //Serial.println(I);  
    
    
     I_PA = (I - ms_lue.OFFSET_I)/ms_lue.G_I;

    
    if (I_PA > ms_lue.lim_I_PA) {                   // on ne veut pas dépasser le seuil de courant PA
      I_PA_max = I_PA;                              // on garde la valeur qui a dépassé le seuil max
      // on coupe le PTT immédiatement 
      Relay_PTT = 0;  // save Relay state
      digitalWrite(6, HIGH); // on coupe le relais PTT
      delay(500);
      Relay_PA = 0;  // save Relay state 
      digitalWrite(7, HIGH); // puis on coupe le relais PA
      alarm_I_PA_Arduino = 1;
      //Serial.println("ordre coupure PTT puis PA Power car I_PA > seuil client");    
    }
    
    // mesure de la puissance directe
    //read_output_CH_1();                           // of module AD8318
    level_CH_1 = calculate_power_CH_1(); 
    if (level_CH_1 > ms_lue.lim_Dir_RF) {         // on ne veut pas dépasser le seuil de puissance directe
      Dir_RF_max = level_CH_1;                    // on garde la valeur qui a dépassé le seuil max
      Relay_PTT = 0;                              // save Relay state
      digitalWrite(6, HIGH);                      // on coupe le relais PTT
      delay(500);
      Relay_PA = 0;                               // save Relay state 
      digitalWrite(7, HIGH);                      // puis on coupe le relais PA
      alarm_Dir_power_Arduino = 1;
      //Serial.println("ordre coupure PTT puis PA Power car Direct Power > seuil max");   
          
    } 
           
    // mesure de la puissance réfléchie
    level_CH_2 = calculate_power_CH_2();
    seuil_R_P_HF = ms_lue.lim_Refl_RF;
    if (calib_R_P_HF) {
      seuil_R_P_HF = 41; // durant la calibration du reflechi on limite le seuil à 41dBm soit 12.5W sinon le seuil en mémoire ne permettrait pas de faire la 2ème mesure de calibration
    }
    
    if (level_CH_2 > seuil_R_P_HF) {              // on ne veut pas dépasser le seuil de puissance réfléchie
      Refl_RF_max = level_CH_2;                   // on garde la valeur qui a dépassé le seuil max
      // on coupe le PTT immédiatement 
      Relay_PTT = 0;                              // save Relay state
      digitalWrite(6, HIGH);                      // on coupe le relais PTT
      delay(500);
      Relay_PA = 0;                               // save Relay state 
      digitalWrite(7, HIGH);                      // puis on coupe le relais PA
      alarm_Refl_power_Arduino = 1;   
    } 
    //Serial.println("*************************FIN MESURES*********************************");
    
}

void loop() { 
    
    Serial.println();
    
    Serial.println("debut loop");
    Serial.print("PA_OFF_ON_LINK_OFF = ");
    Serial.println(PA_OFF_ON_LINK_OFF);
    
    EthernetClient client = server.available();  // try to get client

    mesures();

    if (client) {  // got client?
        boolean currentLineIsBlank = true;
        while (client.connected()) {
            if (client.available()) {   // client data available to read
                char c = client.read(); // read 1 byte (character) from client
                //Serial.print(c);
                
                // limit the size of the stored received HTTP request
                // buffer first part of HTTP request in HTTP_req array (string)
                // leave last element in array as 0 to null terminate string (REQ_BUF_SZ - 1)
                if (req_index < (REQ_BUF_SZ - 1)) {
                    HTTP_req[req_index] = c;          // save HTTP request character
                    req_index++;
                }
                // last line of client request is blank and ends with \n
                // respond to client only after last line received
                if (c == '\n' && currentLineIsBlank) {
                    //Serial.println();
                    // display received HTTP request on serial port
                    //Serial.println(HTTP_req);
                    wdt_reset();  /* Reset the watchdog */
                    // send a standard http response header
                    client.println("HTTP/1.1 200 OK");
                    // remainder of header follows below, depending on if
                    // web page or XML page is requested
                    // Ajax request - send XML file
                    if (StrContains(HTTP_req, "ajax_inputs")) {
                        // send rest of HTTP header
                        client.println("Content-Type: text/xml");
                        client.println("Connection: keep-alive");
                        client.println();

                        // on teste si demande de chargement version longue
                        if (StrContains(HTTP_req, "&advanced")) { // demande de chargement de la page advanced.htm au prochain nouveau client
                          web_page = 1; 
                        }

                        // on teste si demande de chargement version medium
                        if (StrContains(HTTP_req, "&medium")) { // demande de chargement de la page medium.htm au prochain nouveau client
                          web_page = 2;     
                        }

                        // on teste si demande de chargement version courte
                        if (StrContains(HTTP_req, "&short")) { // demande de chargement de la page index.htm au prochain nouveau client
                          web_page = 3;   
                        }
                        
                        
                        // on teste l'action à entreprendre si une coupure de liaison ethernet est détectée
                        Read_req_coupure_PA_on_link_off();
                        
                        // on va tester si une requête des limites est demandée par le client (normalement effectuée à chaque chargement de la page index coté client)
                        Read_req_lim();
                       
                        // commande des relais et du watchdog
                        SetRelays();
                        
                        // on teste si des nouvelles limites sont envoyées par le client
                        if (!req_lim) {
                          // lire les nouvelles limites envoyées par le client et si différentes, les stocker dans l'EEPROM
                          SetLimits();
                        }

                        // on teste si des nouvelles valeurs d'étalonnage sont envoyées par le client si oui on les stocke dans l'EEPROM
                        SetEtalonnage();
                        
                        
                        // reseter éventuellement les alarmes si demandées par le client
                        Reset_alarme();
                        
                        // envoi réponse au client contenant les mesures, l'état des relais, les alarmes ...
                        XML_response(client);
                       
                    }
                    else {  // HTTP request for web page (only once when program start ! )
                      if (!(StrContains(HTTP_req, "favicon"))) {// certains Navigateurs demandent une image icon
                        // il ne faut donc pas envoyer 2 fois la page html ! Merci Yves F4HSL !
                        client.println("Content-Type: text/html");
                        client.println("Connection: keep-alive");
                        client.println();

                        
                        // lire les sorties 41 et 42 pour décider quelle page web sera envoyée au client
                        //if ((digitalRead (41) == LOW) and (digitalRead (42) == LOW))
                        if (web_page == 1)
                        {
                          // send web page advanced.htm - contains version whith diplay, commands and calibration
                          webFile = SD.open("advanced.htm");       // open web page file
                        }
                        //else if ((digitalRead (41) == LOW) and (digitalRead (42) == HIGH)) {
                        else if (web_page == 2) {
                        // send web page index.htm - contains version whith diplay and commands
                          webFile = SD.open("medium.htm");       // open web page file
                        } 
                        else {
                        // send web page index.htm - contains version whith with only display
                          webFile = SD.open("index.htm");       // open web page file
                        }
                       
                          
                        //Serial.println("webFile");
                        if (webFile) {
                            while(webFile.available()) {
                                client.write(webFile.read()); // send web page to client
                            }
                            webFile.close();
                        }
                     }
                     else Serial.println(HTTP_req);
                    }
 
                    // reset buffer index and all buffer elements to 0
                    req_index = 0;
                    StrClear(HTTP_req, REQ_BUF_SZ);
                    break;
                }
                // every line of text received from the client ends with \r\n
                if (c == '\n') {
                    // last character on line of received text
                    // starting new line with next character read
                    currentLineIsBlank = true;
                } 
                else if (c != '\r') {
                    // a text character was received from client
                    currentLineIsBlank = false;
                }
            } // end if (client.available())
        } // end while (client.connected())
        delay(10);      // give the web browser time to receive the data
        client.stop(); // close the connection
    } // end if (client)
}


void SetRelays(void) // lire les commandes du client pour actionner les relais
{
    // Relay 1 (pin 6) // pour le relais PTT
    if (StrContains(HTTP_req, "&PTT=1")) { // on inverse la sortie car la carte relais est active à l'état bas !
        Relay_PTT = 1;  // save Relay state
        digitalWrite(6, LOW);
    }
    else if (StrContains(HTTP_req, "&PTT=0")) {
        Relay_PTT = 0;  // save Relay state
        digitalWrite(6, HIGH);
    }
    // Relay 2 (pin 7) // pour le relais alim secteur PA
    if (StrContains(HTTP_req, "&PA=1")) {
        Relay_PA = 1;  // save Relay state
        digitalWrite(7, LOW);
    }
    else if (StrContains(HTTP_req, "&PA=0")) {
        Relay_PA = 0;  // save Relay state
        digitalWrite(7, HIGH);
        Serial.println("DECODAGE MESSAGE DU CLIENT ' PA POWER OFF ' ");
        Serial.println(HTTP_req);
    }
    // Relay 3 (pin 8) // pour le relais FAN (ventilateurs)
    if (StrContains(HTTP_req, "&FAN=1")) {
        Relay_FAN = 1;  // save Relay state
        digitalWrite(8, LOW);
    }
    else if (StrContains(HTTP_req, "&FAN=0")) {
        Relay_FAN = 0;  // save Relay state
        digitalWrite(8, HIGH);
    }
    // Relay 4 (pin 9) // non utilsé pour l'instant
    if (StrContains(HTTP_req, "&AUX=1")) {
        Relay_AUX = 1;  // save Relay state
        digitalWrite(9, LOW);
    }
    else if (StrContains(HTTP_req, "&AUX=0")) {
        Relay_AUX = 0;  // save Relay state
        digitalWrite(9, HIGH);
    }
   
}

void Reset_alarme() {
  if (StrContains(HTTP_req, "&Reset_alarme")) {
    alarm_T_PA_Arduino = 0;
    alarm_U_PA_Arduino = 0;
    alarm_I_PA_Arduino = 0;
    alarm_Dir_power_Arduino = 0;
    alarm_Refl_power_Arduino = 0;
  }
}

void Read_req_load(void) // on teste si demande de chargement version longue
{
  //if (StrContains(HTTP_req, "&advanced")) req_load = 1; 
  //else req_load = 0; 
}

void Read_req_lim(void) // on teste si une requête de limites est demandée par le client
{
  if (StrContains(HTTP_req, "&str_req_lim")) req_lim = 1; 
  else req_lim = 0; 
}

void Read_req_coupure_PA_on_link_off(void) // on lit quelle action entreprendre sur coupure de la liaison ethernet
{
  if (StrContains(HTTP_req, "&PA_off_on_link_off")) PA_OFF_ON_LINK_OFF = 1;
  if (StrContains(HTTP_req, "&NO_PA_off_on_link_off")) PA_OFF_ON_LINK_OFF = 0;  
}

void SetLimits(void) //
{
    // lire les données stockées en EEPROM
    EEPROM.get(0, ms_lue);
   // Affiche_EEPROM(); // pour debogger !

    //Serial.print("ms.lim_temp_haute = ");
    //Serial.println(ms.lim_temp_haute);
    maj_EEPROM = 0; // pour indiquer qu'il n'y aura pas à priori de mise à jour à faire de l'EEPROM
   
    // lire les éventuelles nouvelles limites envoyées par le client
    
    if (StrContains(HTTP_req, "&Lim_temp_haute=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Lim_temp_haute=");// on récupère le résultat dans la variable globale str_valeur
        ms.lim_temp_haute = (float) atof(str_valeur);
        //Serial.print("ms.lim_temp_haute = ");
        //Serial.println(ms.lim_temp_haute);
        str_valeur[0] = 0;
        // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.lim_temp_haute != ms.lim_temp_haute){
           T_PA_max = 0;
           maj_EEPROM = 1;  
        }
    }
    if (StrContains(HTTP_req, "&Lim_temp_basse=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Lim_temp_basse=");
        ms.lim_temp_basse = (float) atof(str_valeur);
        //Serial.print("ms.lim_temp_basse = ");
        //Serial.println(ms.lim_temp_basse);
        str_valeur[0] = 0;
        // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.lim_temp_basse != ms.lim_temp_basse){
           maj_EEPROM = 1;  
        }
    }
    if (StrContains(HTTP_req, "&Lim_U_PA=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Lim_U_PA=");
        ms.lim_U_PA = (float) atof(str_valeur);
        //Serial.print("ms.lim_U_PA = ");
        //Serial.println(ms.lim_U_PA);
        str_valeur[0] = 0;
        // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.lim_U_PA != ms.lim_U_PA){
           U_PA_max = 0;
           maj_EEPROM = 1;  
        }
    }
    if (StrContains(HTTP_req, "&Lim_I_PA=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Lim_I_PA=");
        ms.lim_I_PA = (float) atof(str_valeur);
        //Serial.print("ms.lim_I_PA = ");
        //Serial.println(ms.lim_I_PA);
        str_valeur[0] = 0;
        // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.lim_I_PA != ms.lim_I_PA){
           I_PA_max = 0;
           maj_EEPROM = 1;  
        }
        
    }
    if (StrContains(HTTP_req, "&Lim_Dir_RF=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Lim_Dir_RF=");
        ms.lim_Dir_RF = (float) atof(str_valeur);
        //Serial.print("ms.lim_Dir_RF = ");
        //Serial.println(ms.lim_Dir_RF);
        str_valeur[0] = 0;
        // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.lim_Dir_RF != ms.lim_Dir_RF){
           Dir_RF_max = 0;
           maj_EEPROM = 1;  
        }
    }
    if (StrContains(HTTP_req, "&Lim_Refl_RF=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Lim_Refl_RF=");
        ms.lim_Refl_RF = (float) atof(str_valeur);
        //Serial.print("ms.lim_Refl_RF = ");
        //Serial.println(ms.lim_Refl_RF);
        str_valeur[0] = 0;
         // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.lim_Refl_RF != ms.lim_Refl_RF){
           Refl_RF_max = 0;
           maj_EEPROM = 1;  
        }
    }

    if (StrContains(HTTP_req, "&Att_Dir_RF=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Att_Dir_RF=");
        ms.Att_D = (float) atof(str_valeur);
        //Serial.print("ms.Att_D = ");
        //Serial.println(ms.Att_D);
        str_valeur[0] = 0;
         // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.Att_D != ms.Att_D){
           //Refl_RF_max = 0;
           maj_EEPROM = 1;  
        }
    }

    if (StrContains(HTTP_req, "&Att_Ref_RF=")) { 
        // on lit la valeur
        StrRead(HTTP_req, "Att_Ref_RF=");
        ms.Att_R = (float) atof(str_valeur);
        //Serial.print("ms.Att_R = ");
        //Serial.println(ms.Att_R);
        str_valeur[0] = 0;
         // tester si la valeur coté client est différente de la valeur en EEPROM
        if (ms_lue.Att_R != ms.Att_R){
           //Refl_RF_max = 0;
           maj_EEPROM = 1;  
        }
    }

    
    if (maj_EEPROM) {
      // faire une mise à jour de l'EEPROM
      //Serial.println("/*****************************************EEPROM.put(0, ms);******************************/");
      EEPROM.put(0, ms);
    }
}


void Affiche_EEPROM() {
  MaStructure m;    
  EEPROM.get(0, m);
  Serial.print("*******************************************magic = ");
  Serial.println(m.magic);
  Serial.print("*******************************************struct_version = ");
  Serial.println(m.struct_version);
  Serial.print("*******************************************lim_temp_haute = ");
  Serial.println(m.lim_temp_haute);
  Serial.print("*******************************************lim_temp_basse = ");
  Serial.println(m.lim_temp_basse);
  Serial.print("*******************************************lim_U_PA = ");
  Serial.println(m.lim_U_PA);
  Serial.print("*******************************************lim_I_PA = ");
  Serial.println(m.lim_I_PA);
  Serial.print("*******************************************lim_Dir_RF = ");
  Serial.println(m.lim_Dir_RF);
  Serial.print("*******************************************lim_Refl_RF = ");
  Serial.println(m.lim_Refl_RF);
  Serial.print("*******************************************G_U_PA = ");
  Serial.println(m.G_U_PA);
  Serial.print("*******************************************G_I = ");
  Serial.println(m.G_I);
  Serial.print("*******************************************OFFSET_I = ");
  Serial.println(m.OFFSET_I);
  Serial.print("*******************************************pente_D = ");
  Serial.println(m.pente_D);
  Serial.print("*******************************************B_D = ");
  Serial.println(m.B_D); 
  Serial.print("*******************************************pente_R = ");
  Serial.println(m.pente_R);
  Serial.print("*******************************************B_R = ");
  Serial.println(m.B_R);
  Serial.print("*******************************************Att_D = ");
  Serial.println(m.Att_D);
  Serial.print("*******************************************Att_R = ");
  Serial.println(m.Att_R); 

}

void SetEtalonnage(void) //
{
    // lire les données stockées en EEPROM
    EEPROM.get(0, ms_lue);

    
   
    // lire les éventuelles nouvelles limites envoyées par le client
    //U_PA = (U * G_U_PA)/100;    
    
    if (StrContains(HTTP_req, "&calib_G_U_PA=")) { 
        float corr_G_U_PA;
        
        // on lit la valeur
        StrRead(HTTP_req, "calib_G_U_PA=");// on récupère le résultat dans la variable globale str_valeur
        corr_G_U_PA = (float) atof(str_valeur);
        //Serial.print("*******************************************corr_G_U_PA = ");
        //Serial.println(corr_G_U_PA);
        str_valeur[0] = 0;
        // calculer la nouvelle valeur de G_P_UA
        G_U_PA = ms_lue.G_U_PA * corr_G_U_PA;
        ms.G_U_PA = G_U_PA;
        //Serial.print("*******************************************nouveau G_U_PA = ");
        //Serial.println(ms.G_U_PA);
        if ((ms.G_U_PA < 0.996 * ms_lue.G_U_PA) || (ms.G_U_PA > 1.0036 * ms_lue.G_U_PA)){ 
          //Affiche_EEPROM();
             //Serial.println("/*****************************************EEPROM.put(0, ms);******************************/");
             EEPROM.put(0, ms); 
          //Affiche_EEPROM();   
          }
        
    }
    
    if (StrContains(HTTP_req, "&calib_I=")) { 
        float C1,C0,C;
        float G1,G0,G;
        // on lit la valeur
        /*
        StrRead(HTTP_req, "I_zero_lu=");
        Serial.print("*******************************************str_valeur  = ");
        Serial.println(str_valeur);
         */
        StrRead_2val(HTTP_req, "calib_I=");
        //Serial.print("*******************************************str_valeur  = ");
        //Serial.println(str_valeur);
        //Serial.print("*******************************************str_valeur2  = ");
        //Serial.println(str_valeur2);
        // calcul nouvel offset
        C1 = (float) atof(str_valeur);      // correction d'offset d'après la mesure de calibration
        G1 = (float) atof(str_valeur2);     // correction de gain d'après la mesure de calibration
        C0 = ms_lue.OFFSET_I;               // offset avant calbration
        G0 = ms_lue.G_I;                    // gain avant calibration
        C = C0 + C1*G0;                     // nouvel offset
        G = G0 * G1;                        // nouveau gain
        ms.OFFSET_I = C;
        ms.G_I = G;
        str_valeur[0] = 0;
        str_valeur2[0] = 0;
        
        //Serial.print("*******************************************nouveau OFFSET_I = ");
        //Serial.println(ms.OFFSET_I);
        //Serial.print("**********************************************ms.G_I = ");
        //Serial.println(ms.G_I);
        // tester si l'offset ou le gain sont suffisemment différents des valeurs stockées en EEPROM
        if (((ms.OFFSET_I < 0.99 * ms_lue.OFFSET_I) || (ms.OFFSET_I > 1.01 * ms_lue.OFFSET_I))
        || ((ms.G_I < 0.99 * ms_lue.G_I) || (ms.G_I > 1.01 * ms_lue.G_I)))
        {
           //Serial.println("/*****************************************EEPROM.put(0, ms);******************************/");
           EEPROM.put(0, ms);
        }

        
    }
/*
    if (StrContains(HTTP_req, "&debut_cal_D_P_HF")) { 
        calib_D_P_HF = true;        
    }
*/

    if (StrContains(HTTP_req, "&param_cal_D_P_HF=")) { 

        // récupération des paramètres de correction DPS1,DPS2,DPL1,DPL2
        StrRead_4val(HTTP_req, "&param_cal_D_P_HF=");
        float DPS1 = (float) atof(str_valeur);      
        float DPS2 = (float) atof(str_valeur2);
        float DPL1 = (float) atof(str_valeur3);      
        float DPL2 = (float) atof(str_valeur4);
        //Serial.print("*******************************************DPS1 = ");
        //Serial.println(DPS1);
        //Serial.print("*******************************************DPS2 = ");
        //Serial.println(DPS2);
        //Serial.print("*******************************************DPL1 = ");
        //Serial.println(DPL1);
        //Serial.print("*******************************************DPL2 = ");
        //Serial.println(DPL2);
        
        str_valeur[0] = 0;
        str_valeur2[0] = 0;
        str_valeur3[0] = 0;
        str_valeur4[0] = 0;
        
        EEPROM.get(0, ms_Adjust_HF);
        // calcul de la nouvelle pente
        float G = (DPL2 - DPL1) / (DPS2 - DPS1);
        float pente = ms_Adjust_HF.pente_D;    // ancienne pente
        float B = ms_Adjust_HF.B_D;
        float new_pente = pente * G;   // nouvelle pente
        //calcul du nouveau B
        float att_CH1 = ms.Att_D;;
        float new_B = (DPS1 * new_pente) - (DPL1 * pente) + att_CH1 * (pente - new_pente) + B; // nouveau B
        
        //Serial.print(" new_pente = ");Serial.print(new_pente);Serial.print(" new_B = ");Serial.println(new_B);
        
        ms_Adjust_HF.pente_D = new_pente;
        ms_Adjust_HF.B_D = new_B;

        // tester si la pente ou B sont suffisemment différents des valeurs stockées en EEPROM
        if (((new_pente < 0.99 * pente) || (new_pente > 1.01 * pente))
        || ((new_B < 0.99 * B) || (new_B > 1.01 * B)))
        { // écrire ces valeurs dans l'EEPROM
         Affiche_EEPROM(); 
            //Serial.print("****************************** MAJ EEPROM pente et B Puissance directe*******************");
            //Serial.println("*************************************EEPROM.put(0, ms);******************************");
            EEPROM.put(0, ms_Adjust_HF);
         Affiche_EEPROM();  
        }

    }
    
    if (StrContains(HTTP_req, "&debut_cal_R_P_HF")) { 
        calib_R_P_HF = true; 
    }
        
    if (StrContains(HTTP_req, "&param_cal_R_P_HF=")) { 

        // récupération des paramètres de correction RPS1,RPS2,RPL1,RPL2
        StrRead_4val(HTTP_req, "&param_cal_R_P_HF=");
        float RPS1 = (float) atof(str_valeur);      
        float RPS2 = (float) atof(str_valeur2);
        float RPL1 = (float) atof(str_valeur3);      
        float RPL2 = (float) atof(str_valeur4);
        //Serial.print("*******************************************RPS1 = ");
        //Serial.println(RPS1);
        //Serial.print("*******************************************RPS2 = ");
        //Serial.println(RPS2);
        //Serial.print("*******************************************RPL1 = ");
        //Serial.println(RPL1);
        //Serial.print("*******************************************RPL2 = ");
        //Serial.println(RPL2);
        
        str_valeur[0] = 0;
        str_valeur2[0] = 0;
        str_valeur3[0] = 0;
        str_valeur4[0] = 0;
        
        EEPROM.get(0, ms_Adjust_HF);
        // calcul de la nouvelle pente
        float G = (RPL2 - RPL1) / (RPS2 - RPS1);
        float pente = ms_Adjust_HF.pente_R;    // ancienne pente
        float B = ms_Adjust_HF.B_R;
        float new_pente = pente * G;   // nouvelle pente
        //calcul du nouveau B
        float att_CH2 = ms.Att_R;;
        float new_B = (RPS1 * new_pente) - (RPL1 * pente) + att_CH2 * (pente - new_pente) + B; // nouveau B
        //Serial.println("*****************************************************************************************************/");
        //Serial.print(" pente reflechi = ");Serial.print(pente);Serial.print(" B reflechi = ");Serial.println(B);
        
        //Serial.print(" new_pente reflechi = ");Serial.print(new_pente);Serial.print(" new_B reflechi = ");Serial.println(new_B);
        //Serial.println("*****************************************************************************************************/");
        ms_Adjust_HF.pente_R = new_pente;
        ms_Adjust_HF.B_R = new_B;

        // tester si la pente ou B sont suffisemment différents des valeurs stockées en EEPROM
        if (((new_pente < 0.99 * pente) || (new_pente > 1.01 * pente))
        || ((new_B < 0.99 * B) || (new_B > 1.01 * B)))
        { // écrire ces valeurs dans l'EEPROM
         Affiche_EEPROM(); 
            //Serial.print("*********************** MAJ EEPROM pente et B puissance Reflechie*******************");
            //Serial.println("**********************************EEPROM.put(0, ms);******************************");
            EEPROM.put(0, ms_Adjust_HF);
         Affiche_EEPROM();  
        }

    }
    if (StrContains(HTTP_req, "&fin_cal_R_P_HF")) { 
        calib_R_P_HF = false; 
    }    

}

// send the XML file with analog values, switch status
// and Relays status
void XML_response(EthernetClient cl)
{
    int analog_val;            // stores value read from analog inputs
    int count;                 // used by 'for' loops
    int sw_arr[] = {2, 3, 5};  // pins interfaced to switches NON utilsés pour l'instant
        
    cl.print("<?xml version = \"1.0\" ?>");
    cl.print("<inputs>");
    // envoi des mesures
    cl.print("<temp>");
    cl.print(DS18B20_temperature); 
    cl.println("</temp>");
    cl.print("<U_PA>");
    cl.print(U_PA,1);
    cl.println("</U_PA>");
    cl.print("<I_PA>");
    cl.print(I_PA,1);
    cl.println("</I_PA>"); 
    cl.print("<Dir_RF>");
    cl.print(level_CH_1);
    cl.println("</Dir_RF>");
    cl.print("<Refl_RF>");
    cl.print(level_CH_2);
    cl.println("</Refl_RF>"); 

    // envoi états des relais
    
    // etat relais PTT
    cl.print("<etat_PTT>");
    if (Relay_PTT) {
        cl.print("on");
    }
    else {
        cl.print("off");
    }
    cl.println("</etat_PTT>");

    // etat relais PA
    cl.print("<etat_PA>");
    if (Relay_PA == 1) {
        cl.print("on");
    }
    else if (Relay_PA == 0) {
        cl.print("off");
    }
     cl.println("</etat_PA>");

    // etat relais FAN
    cl.print("<etat_FAN>");
    //Serial.print("<Relay>");
    if (Relay_FAN) {
        cl.print("on");
    }
    else {
        cl.print("off");
    }
    cl.println("</etat_FAN>");

    // etat relais AUX
    cl.print("<etat_AUX>");
    //Serial.print("<Relay>");
    if (Relay_AUX) {
        cl.print("on");
    }
    else {
        cl.print("off");
    }
    cl.println("</etat_AUX>");
    
    //Envoi du  Watchdog : on envoie alternativement des messages Watchdog on et off au client pour qu'il puisse vérifier que la liaison ethernet fonctionne de son coté
    cl.print("<Watchdog>");
    if (Watchdog) {
        Watchdog = 0;
        cl.print("off");
        Serial.print("off");
    }
    else {
        Watchdog = 1;    
        cl.print("on");
        Serial.print("on");
    }
    cl.println("</Watchdog>");

    //Envoi du type de page programmé pour la prochaine connexion
    cl.print("<web_page>");
    if (web_page == 1) {
      // will send web page advanced.htm on next connexion
      cl.print("full");
    }
    else if (web_page == 2) {
    // will send web page medium.htm on next connexion
      cl.print("medium");
    } 
    else {
      // will send web page index.htm on next connexion
      cl.print("short");       
    }
    cl.println("</web_page>");
 
    // envoi alarm_T_PA_Arduino au client
    
    if (alarm_T_PA_Arduino) {
      cl.print("<alarm_T_PA_Arduino>");
      //Serial.print("<alarm_T_PA_Arduino>");
      cl.print(T_PA_max);
      //Serial.print(T_PA_max);
      cl.println("</alarm_T_PA_Arduino>");
      //Serial.print("</alarm_T_PA_Arduino>");
    }   
    
    // envoi alarm_U_PA_Arduino au client
    
    if (alarm_U_PA_Arduino) {
      cl.print("<alarm_U_PA_Arduino>");
      //Serial.print("<alarm_U_PA_Arduino>");
      cl.print(U_PA_max);
      //Serial.print(U_PA_max);
      cl.println("</alarm_U_PA_Arduino>");
      //Serial.print("</alarm_U_PA_Arduino>");
    }
       
    // envoi alarm_I_PA_Arduino au client
    
    if (alarm_I_PA_Arduino) {
      cl.print("<alarm_I_PA_Arduino>");
      //Serial.print("<alarm_I_PA_Arduino>");
      cl.print(I_PA_max);
      //Serial.print(I_PA_max);
      cl.println("</alarm_I_PA_Arduino>");
      //Serial.println("</alarm_I_PA_Arduino>");
    }

    // envoi alarm_Dir_power_Arduino au client
    
    if (alarm_Dir_power_Arduino) {
      cl.print("<alarm_Dir_power_Arduino>");
      //Serial.print("<alarm_Dir_power_Arduino>");
      cl.print(Dir_RF_max);
      //Serial.print(Dir_RF_max);
      cl.println("</alarm_Dir_power_Arduino>");
      //Serial.println("</alarm_Dir_power_Arduino>");
    }
    
    // envoi alarm_Refl_power_Arduino au client
    if (alarm_Refl_power_Arduino) {
       cl.print("<alarm_Refl_power_Arduino>");
       //Serial.print("<alarm_Refl_power_Arduino>");
       cl.print(Refl_RF_max);
       //Serial.print(Refl_RF_max);
       cl.println("</alarm_Refl_power_Arduino>");
       //Serial.println("</alarm_Refl_power_Arduino>");
    }

    if (req_lim == 1) { // si requete des limites par le client
      // envoyer les limites au client
       Serial.println("envoi des limites vers le client !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
      cl.print("<lim_T_H>");
      cl.print(ms.lim_temp_haute,1);
      cl.print("</lim_T_H>");
      cl.print("<lim_T_B>");
      cl.print(ms.lim_temp_basse,1);
      cl.print("</lim_T_B>");
      cl.print("<lim_U_PA>");
      cl.print(ms.lim_U_PA,1);
      cl.print("</lim_U_PA>");
      cl.print("<lim_I_PA>");
      cl.print(ms.lim_I_PA,1);
      cl.print("</lim_I_PA>");
      cl.print("<lim_Dir_RF>");
      cl.print(ms.lim_Dir_RF,1);
      cl.print("</lim_Dir_RF>");
      cl.print("<lim_Refl_RF>");
      cl.print(ms.lim_Refl_RF,1);
      cl.print("</lim_Refl_RF>");
      
      cl.print("<Att_Dir_RF>");
      cl.print(ms.Att_D,1);
      cl.print("</Att_Dir_RF>");
      
      cl.print("<Att_Ref_RF>");
      cl.print(ms.Att_R,1);
      cl.print("</Att_Ref_RF>");


      
      req_lim = 0;
    }
    cl.print("</inputs>");    
}

// sets every element of str to 0 (clears array)
void StrClear(char *str, char length)
{
    for (int i = 0; i < length; i++) {
        str[i] = 0;
    }
}

// searches for the string sfind in the string str
// returns 1 if string found
// returns 0 if string not found
char StrContains(char *str, char *sfind)
{
    char found = 0;
    char index = 0;
    char len;

    len = strlen(str);
    
    if (strlen(sfind) > len) {
        return 0;
    }
    while (index < len) {
        if (str[index] == sfind[found]) {
            found++;
            if (strlen(sfind) == found) {
                return 1;
            }
        }
        else {
            found = 0;
        }
        index++;
    }

    return 0;
}

void StrRead(char *str, char *sfind) // extrait la valeur numérique de la chaine sfind dans la chaine str et la recopie dans la chaine globale str_valeur
{
    str_valeur[0]=0;
    char i = 0; 
    char j = 0; 
    char * pch;
    pch = strstr (str,sfind); 
    while (pch[i] != '=') {
      i++;
    }
    i++;   
    while (pch[i] != '&') {
      str_valeur[j] = pch[i] ;
      i++;
      j++;
    }
    str_valeur[j]=0;
}

void StrRead_2val(char *str, char *sfind) // extrait 2 valeurs numériques de la chaine sfind dans la chaine str et la recopie dans les 2 chaine globales str_valeur et str_valeur2
{
    str_valeur[0]=0;
    str_valeur2[0]=0;
    char i = 0; 
    char j = 0; 
    char * pch;
    pch = strstr (str,sfind); //renvoie un pointeur sur le début de sfind
    while (pch[i] != '=') {
      i++;
    }
    i++;   
    while (pch[i] != '/') {
      str_valeur[j] = pch[i] ;
      i++;
      j++;
    }
    i++;    // on pointe juste après le /
    str_valeur[j]=0;
    j=0;
    while (pch[i] != '&') {
      str_valeur2[j] = pch[i] ;
      i++;
      j++;
    }
    str_valeur2[j]=0;    
}

void StrRead_4val(char *str, char *sfind) // extrait 4 valeurs numériques de la chaine sfind dans la chaine str 
                                          // et la recopie dans les 4 chaine globales str_valeur, str_valeur2, str_valeur3, str_valeur4
                                          // les 4 valeurs sont séparées par '/' la dernière valeur se termine par '&'
{
    str_valeur[0]=0;
    str_valeur2[0]=0;
    str_valeur3[0]=0;
    str_valeur4[0]=0;
    char i = 0; 
    char j = 0; 
    char * pch;
    pch = strstr (str,sfind); //renvoie un pointeur sur le début de sfind
    while (pch[i] != '=') {
      i++;
    }
    
    i++;     // on pointe juste après le =  
    while (pch[i] != '/') {
      str_valeur[j] = pch[i] ;
      i++;
      j++;
    }
    str_valeur[j]=0;

    // recherche du DPS2
    i++;    // on pointe juste après le /
    j=0;
    while (pch[i] != '/') {
      str_valeur2[j] = pch[i] ;
      i++;
      j++;
    }
    str_valeur2[j]=0; 

    // recherche de DPL1
    i++;    // on pointe juste après le /
    j=0;
    while (pch[i] != '/') {
      str_valeur3[j] = pch[i] ;
      i++;
      j++;
    }
    str_valeur3[j]=0; 

    // recherche de DPL2
    i++;    // on pointe juste après le /    
    j=0;
    while (pch[i] != '&') {
      str_valeur4[j] = pch[i] ;
      i++;
      j++;
    }
    str_valeur4[j]=0;    
}

/* --------------- Acquisition de la température ----------------------------------- */
float getTemperatureDS18b20(){
  byte i;
  byte data[12];
  byte addr[8];
  float temp =0.0;
  
  //Il n'y a qu'un seul capteur, donc on charge l'unique adresse.
  ds.search(addr);
  
  // Cette fonction sert à surveiller si la transmission s'est bien passée
  if (OneWire::crc8( addr, 7) != addr[7]) {
    Serial.println("getTemperatureDS18b20 : <!> CRC is not valid! <!>");
    return false;
  }
  
  // On vérifie que l'élément trouvé est bien un DS18B20
  if (addr[0] != DS18B20_ID) {
    Serial.println("L'équipement trouvé n'est pas un DS18B20");
    return false;
  }
  
  // programmer une résolution de 9 bits
  changeResolution( addr, RESOLUTION_9_BITS);
  
  // Demander au capteur de mémoriser la température et lui laisser 200ms pour le faire (voir datasheet)
  ds.reset();
  ds.select(addr);
  ds.write(0x44);
  delay(200);
  // Demander au capteur de nous envoyer la température mémorisée
  ds.reset();
  ds.select(addr);
  ds.write(0xBE);
  
  // Le MOT reçu du capteur fait 9 octets, on les charge donc un par un dans le tableau data[]
  for ( i = 0; i < 9; i++) {
   data[i] = ds.read();
  }
  // Puis on converti la température (*0.0625 car la température est stockée sur 12 bits)
  temp = ( (data[1] << 8) + data[0] )*0.0625;
  
  return temp;
}


/** Change la résolution du capteur de température */
void changeResolution(const byte addr[], byte resolution) {

  /* Reset le bus 1-Wire, sélectionne le capteur et envoie une demande d'écriture du scratchpad */
  ds.reset();
  ds.select(addr);
  ds.write(0x4E);
  
  /* Ecrit dans le scratchpad */
  ds.write(0x00);
  ds.write(0x00);
  ds.write(resolution);

  /* Fin d'écriture */
  ds.reset();
}


// -----------------------------------------------------------------------------------------------------

float calculate_power_CH_1()                              // equation d'une droite : y = pente*x + B => x = (y - B)/pente
{  
  float voltage_CH_1  =  analogRead(A4);                  // read 1x
  for(iii=0;iii<199; iii++)                               // iii = counter index      
      voltage_CH_1  +=  analogRead(A4);                   // read 199x
  voltage_CH_1  =  voltage_CH_1/40920;                    // calculate average value of 200 readings (40920 = 200*1023/5V)                             
  //Serial.print("voltage_CH_1 = ");
  //Serial.println(voltage_CH_1); 
  Affiche_EEPROM();
  // lecture B et pente dans l'EEPROM
  EEPROM.get(0, ms_lue);
  float B = ms_lue.B_D;
  float pente = ms_lue.pente_D;
  float att_CH1 = ms_lue.Att_D;                           // 38db coupleur interne au PA et 30 db coupleurs entrée du  AC 8318
  float Level_CH_1 = (B - 1000*voltage_CH_1)/pente + att_CH1;  // niveau en dBm      
  //Serial.print("  Level_CH_1 = ");
  //Serial.println(Level_CH_1);
  if (Level_CH_1 < 25 ) Level_CH_1 = -50;                 // on est plus dans la zone linéaire de l'AD 8318 !                   
  //float power_W_1 = pow(10,level_CH_1/10);              // convert dBm to mW 
  return Level_CH_1;
}

// -----------------------------------------------------------------------------------------------------
float  calculate_power_CH_2()                                // equation d'une droite : y = pente*x + B => x = (y - B)/pente
{ 
  float voltage_CH_2  =  analogRead(A5);                     // read 1x
  for(iii=0;iii<199; iii++)                                  // iii = counter index      
    voltage_CH_2  +=  analogRead(A5);                        // read 199x
  voltage_CH_2  =  voltage_CH_2/40920;                       // calculate average value of 200 readings ( 40920 = 200*1023/5V ) 
  //Serial.print("  voltage_CH_2 = ");
  //Serial.println(voltage_CH_2);    
  // lecture B et pente dans l'EEPROM
  EEPROM.get(0, ms_lue);
  float B = ms_lue.B_R;
  float pente = ms_lue.pente_R;
  float att_CH2 = ms_lue.Att_R;                              // 40db coupleur externe au PA branché pour le réfléchi et 20 db coupleurs entrée du  AC 8318  
  //Serial.print("  B = ");
  //Serial.println(B); 
  //Serial.print("  pente = ");
  //Serial.println(pente); 
  //Serial.print("  att_CH2 = ");
  //Serial.println(att_CH2); 
  float Level_CH_2 = (B - 1000*voltage_CH_2)/pente + att_CH2;  // niveau en dBm                                
  //Serial.print("  Level_CH_2 = ");
  //Serial.println(Level_CH_2);  
  if (Level_CH_2 < 15 ) Level_CH_2 = -50;                   // on est plus dans la zone linéaire de l'AD 8318 !
  //float power_W_2 = pow(10,level_CH_2/10);                // convert dBm to mW 
  return Level_CH_2;
}
// -----------------------------------------------------------------------------------------------------
